"""The diag commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - DIAg:CONTROL:HALT {ON|OFF|<NR1>}
    - DIAg:CONTROL:HALT?
    - DIAg:CONTROL:LOOP {ON|OFF|<NR1>}
    - DIAg:CONTROL:LOOP?
    - DIAg:EXECUTE
    - DIAg:FAILURES:CLEAR {ON|OFF|<NR1>}
    - DIAg:FAILURES:CLEAR?
    - DIAg:ITEM:FAILURES? <NR1>
    - DIAg:ITEM:NAMe? <NR1>
    - DIAg:ITEM:RESULT? <NR1>
    - DIAg:ITEM:SUBITEMS? <NR1>
    - DIAg:ITEM? <NR1>
    - DIAg:LEVEL {AREA|SUBSYS|TEST}
    - DIAg:LEVEL?
    - DIAg:LOOPS?
    - DIAg:NAMe:AREA?
    - DIAg:NAMe:SUBSYS?
    - DIAg:NAMe:TEST?
    - DIAg:NAMe?
    - DIAg:NUMITEMS?
    - DIAg:RESults:VERBose?
    - DIAg:RESults?
    - DIAg:SELect:ALL ALL
    - DIAg:SELect:AREA <NR1>
    - DIAg:SELect:AREA?
    - DIAg:SELect:LAST <NR1>
    - DIAg:SELect:LAST?
    - DIAg:SELect:SUBSYS <NR1>
    - DIAg:SELect:SUBSYS?
    - DIAg:SELect:TEST <NR1>
    - DIAg:SELect:TEST?
    - DIAg:STATE {<NR1>|EXECUTE|ON|OFF}
    - DIAg:STATE?
    - DIAg:STOP
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdReadWithArguments, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class DiagStop(SCPICmdWriteNoArguments):
    """The ``DIAg:STOP`` command.

    Description:
        - This command (no query form) causes diagnostics (or test) execution to terminate at the
          end of the next low-level test. This command is equivalent to selecting Instrument
          Diagnostics from the Utilities menu and then clicking Abort.

    Usage:
        - Using the ``.write()`` method will send the ``DIAg:STOP`` command.

    SCPI Syntax:
        ```
        - DIAg:STOP
        ```
    """


class DiagState(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:STATE`` command.

    Description:
        - This command changes the instrument operating state. Depending on the argument,
          diagnostics capabilities are either turned on or off. This command is equivalent to
          opening the ``DIAg:STATE`` dialog (ON) or closing it (OFF).

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:STATE value`` command.

    SCPI Syntax:
        ```
        - DIAg:STATE {<NR1>|EXECUTE|ON|OFF}
        - DIAg:STATE?
        ```

    Info:
        - ``<NR1>`` = 0 disables diagnostics capabilities and returns the instrument to a normal
          operating state; any other value enables diagnostics.
        - ``EXECUTE`` starts execution of the diagnostics.
        - ``ON`` puts the instrument into the state in which diagnostics can be run. This argument
          is thrown automatically if the ``DIAG:EXECUTE`` command is executed.
        - ``OFF`` disables diagnostics capabilities and returns the instrument to a normal operating
          state.
    """


class DiagSelectTest(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:SELect:TEST`` command.

    Description:
        - This command selects or queries one of the available tests. This command is equivalent to
          selecting Instrument Diagnostics from the Utilities menu and then choosing a Test from the
          drop-down list.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:SELect:TEST?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:SELect:TEST?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:SELect:TEST value`` command.

    SCPI Syntax:
        ```
        - DIAg:SELect:TEST <NR1>
        - DIAg:SELect:TEST?
        ```

    Info:
        - ``<NR1>`` selects a test by number, which can range from 0 (zero selects ALL) through 15
          (as limited by the return from ``DIAG:NUMITEMS``).
    """


class DiagSelectSubsys(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:SELect:SUBSYS`` command.

    Description:
        - This command selects or queries the available subsystem. This command is equivalent to
          selecting Instrument Diagnostics from the Utilities menu and then choosing a Subsystem
          from the drop-down list.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:SELect:SUBSYS?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:SELect:SUBSYS?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:SELect:SUBSYS value`` command.

    SCPI Syntax:
        ```
        - DIAg:SELect:SUBSYS <NR1>
        - DIAg:SELect:SUBSYS?
        ```

    Info:
        - ``<NR1>`` selects a subsystem by number, which can range from 0 (zero selects ALL) through
          15 (as limited by the return from ``DIAG:NUMITEMS``).
    """


class DiagSelectLast(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:SELect:LAST`` command.

    Description:
        - This command selects one or more diagnostic menu items to be executed via the
          ``DIAG:EXECUTE`` command. If you specify ``DIAg:LEVEL SUBSYS``, then menu items come from
          this diagnostic level and are limited to the value returned by the ``DIAg:NUMITEMS?``
          query. For example, if the return from the ``DIAg:NUMITEMS?`` query is
          ``:DIAG:NUMITEMS 5``, specifying ``DIAg:SELECT:SUBSYS 2`` indicates that diagnostics will
          start from subsystem 2 and that you can specify a range from 2 through 5 for
          ``DIAg:SELect:LAST``. If you enter ``:DIAg:SELect:LAST 2``, only subsystem 2 will be
          executed. ``DIAg:SELect:LAST 4``, subsystems 2 through 4 will be executed.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:SELect:LAST?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:SELect:LAST?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:SELect:LAST value`` command.

    SCPI Syntax:
        ```
        - DIAg:SELect:LAST <NR1>
        - DIAg:SELect:LAST?
        ```

    Info:
        - ``<NR1>`` selects an integer that identifies the number of the last item that will be
          executed when the ``DIAG:EXECUTE`` command is run.
    """


class DiagSelectArea(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:SELect:AREA`` command.

    Description:
        - This command selects or queries an available diagnostic area. This command is equivalent
          to selecting Instrument Diagnostics from the Utilities menu and then selecting an Area
          from the pull-down list.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:SELect:AREA?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:SELect:AREA?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:SELect:AREA value`` command.

    SCPI Syntax:
        ```
        - DIAg:SELect:AREA <NR1>
        - DIAg:SELect:AREA?
        ```

    Info:
        - ``<NR1>`` selects a diagnostic area by number, which can range from 0 (zero selects all)
          through 15 (as specified by ``DIAG:NUMITEMS``).
    """


class DiagSelectAll(SCPICmdWrite):
    """The ``DIAg:SELect:ALL`` command.

    Description:
        - This command (no query form) selects all available diagnostics. This command is equivalent
          to selecting Instrument Diagnostics from the Utilities menu and then choosing ALL from the
          Subsystem, Area and Test pull-down lists.

    Usage:
        - Using the ``.write(value)`` method will send the ``DIAg:SELect:ALL value`` command.

    SCPI Syntax:
        ```
        - DIAg:SELect:ALL ALL
        ```

    Info:
        - ``ALL`` selects all available diagnostics.
    """


class DiagSelect(SCPICmdRead):
    """The ``DIAg:SELect`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:SELect?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:SELect?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.all``: The ``DIAg:SELect:ALL`` command.
        - ``.area``: The ``DIAg:SELect:AREA`` command.
        - ``.last``: The ``DIAg:SELect:LAST`` command.
        - ``.subsys``: The ``DIAg:SELect:SUBSYS`` command.
        - ``.test``: The ``DIAg:SELect:TEST`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._all = DiagSelectAll(device, f"{self._cmd_syntax}:ALL")
        self._area = DiagSelectArea(device, f"{self._cmd_syntax}:AREA")
        self._last = DiagSelectLast(device, f"{self._cmd_syntax}:LAST")
        self._subsys = DiagSelectSubsys(device, f"{self._cmd_syntax}:SUBSYS")
        self._test = DiagSelectTest(device, f"{self._cmd_syntax}:TEST")

    @property
    def all(self) -> DiagSelectAll:
        """Return the ``DIAg:SELect:ALL`` command.

        Description:
            - This command (no query form) selects all available diagnostics. This command is
              equivalent to selecting Instrument Diagnostics from the Utilities menu and then
              choosing ALL from the Subsystem, Area and Test pull-down lists.

        Usage:
            - Using the ``.write(value)`` method will send the ``DIAg:SELect:ALL value`` command.

        SCPI Syntax:
            ```
            - DIAg:SELect:ALL ALL
            ```

        Info:
            - ``ALL`` selects all available diagnostics.
        """
        return self._all

    @property
    def area(self) -> DiagSelectArea:
        """Return the ``DIAg:SELect:AREA`` command.

        Description:
            - This command selects or queries an available diagnostic area. This command is
              equivalent to selecting Instrument Diagnostics from the Utilities menu and then
              selecting an Area from the pull-down list.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:SELect:AREA?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:SELect:AREA?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:SELect:AREA value`` command.

        SCPI Syntax:
            ```
            - DIAg:SELect:AREA <NR1>
            - DIAg:SELect:AREA?
            ```

        Info:
            - ``<NR1>`` selects a diagnostic area by number, which can range from 0 (zero selects
              all) through 15 (as specified by ``DIAG:NUMITEMS``).
        """
        return self._area

    @property
    def last(self) -> DiagSelectLast:
        """Return the ``DIAg:SELect:LAST`` command.

        Description:
            - This command selects one or more diagnostic menu items to be executed via the
              ``DIAG:EXECUTE`` command. If you specify ``DIAg:LEVEL SUBSYS``, then menu items come
              from this diagnostic level and are limited to the value returned by the
              ``DIAg:NUMITEMS?`` query. For example, if the return from the ``DIAg:NUMITEMS?`` query
              is ``:DIAG:NUMITEMS 5``, specifying ``DIAg:SELECT:SUBSYS 2`` indicates that
              diagnostics will start from subsystem 2 and that you can specify a range from 2
              through 5 for ``DIAg:SELect:LAST``. If you enter ``:DIAg:SELect:LAST 2``, only
              subsystem 2 will be executed. ``DIAg:SELect:LAST 4``, subsystems 2 through 4 will be
              executed.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:SELect:LAST?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:SELect:LAST?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:SELect:LAST value`` command.

        SCPI Syntax:
            ```
            - DIAg:SELect:LAST <NR1>
            - DIAg:SELect:LAST?
            ```

        Info:
            - ``<NR1>`` selects an integer that identifies the number of the last item that will be
              executed when the ``DIAG:EXECUTE`` command is run.
        """
        return self._last

    @property
    def subsys(self) -> DiagSelectSubsys:
        """Return the ``DIAg:SELect:SUBSYS`` command.

        Description:
            - This command selects or queries the available subsystem. This command is equivalent to
              selecting Instrument Diagnostics from the Utilities menu and then choosing a Subsystem
              from the drop-down list.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:SELect:SUBSYS?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:SELect:SUBSYS?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:SELect:SUBSYS value`` command.

        SCPI Syntax:
            ```
            - DIAg:SELect:SUBSYS <NR1>
            - DIAg:SELect:SUBSYS?
            ```

        Info:
            - ``<NR1>`` selects a subsystem by number, which can range from 0 (zero selects ALL)
              through 15 (as limited by the return from ``DIAG:NUMITEMS``).
        """
        return self._subsys

    @property
    def test(self) -> DiagSelectTest:
        """Return the ``DIAg:SELect:TEST`` command.

        Description:
            - This command selects or queries one of the available tests. This command is equivalent
              to selecting Instrument Diagnostics from the Utilities menu and then choosing a Test
              from the drop-down list.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:SELect:TEST?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:SELect:TEST?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:SELect:TEST value`` command.

        SCPI Syntax:
            ```
            - DIAg:SELect:TEST <NR1>
            - DIAg:SELect:TEST?
            ```

        Info:
            - ``<NR1>`` selects a test by number, which can range from 0 (zero selects ALL) through
              15 (as limited by the return from ``DIAG:NUMITEMS``).
        """
        return self._test


class DiagResultsVerbose(SCPICmdRead):
    """The ``DIAg:RESults:VERBose`` command.

    Description:
        - This query-only command returns a more explanatory message about the results of the last
          diagnostic (or test) execution than the ``DIAg:RESults?`` query. This command is
          equivalent to selecting Instrument Diagnostics from the Utilities menu and then reviewing
          the Diagnostic Status. This query-only command can be issued while diagnostics are still
          in progress.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:RESults:VERBose?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:RESults:VERBose?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:RESults:VERBose?
        ```
    """


class DiagResults(SCPICmdRead):
    """The ``DIAg:RESults`` command.

    Description:
        - This query-only command returns an abbreviated status about the results of the last
          diagnostic (or test) execution. For a more explanatory status message, use the
          ``DIAg:RESults:VERBose?`` query. This command is equivalent to selecting Instrument
          Diagnostics from the Utilities menu and then reviewing the Diagnostic Status. This
          query-only command can be issued while diagnostics are still in progress.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:RESults?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:RESults?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:RESults?
        ```

    Properties:
        - ``.verbose``: The ``DIAg:RESults:VERBose`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._verbose = DiagResultsVerbose(device, f"{self._cmd_syntax}:VERBose")

    @property
    def verbose(self) -> DiagResultsVerbose:
        """Return the ``DIAg:RESults:VERBose`` command.

        Description:
            - This query-only command returns a more explanatory message about the results of the
              last diagnostic (or test) execution than the ``DIAg:RESults?`` query. This command is
              equivalent to selecting Instrument Diagnostics from the Utilities menu and then
              reviewing the Diagnostic Status. This query-only command can be issued while
              diagnostics are still in progress.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:RESults:VERBose?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:RESults:VERBose?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:RESults:VERBose?
            ```
        """
        return self._verbose


class DiagNumitems(SCPICmdRead):
    """The ``DIAg:NUMITEMS`` command.

    Description:
        - This query-only command returns the number of items on the currently selected level of
          test hierarchy, which ranges from 1 through 15. This command is equivalent to selecting
          Instrument Diagnostics from the Utilities menu and then reviewing the Diagnostic Status.
          This query-only command can be issued while diagnostics are still in progress.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:NUMITEMS?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:NUMITEMS?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:NUMITEMS?
        ```
    """


class DiagNameTest(SCPICmdRead):
    """The ``DIAg:NAMe:TEST`` command.

    Description:
        - This query-only command returns the name of the current diagnostic test. This command is
          equivalent to selecting Instrument Diagnostics from the Utilities menu and then reviewing
          the Diagnostic Status. This query-only command can be issued while diagnostics are still
          in progress.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:NAMe:TEST?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:NAMe:TEST?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:NAMe:TEST?
        ```
    """


class DiagNameSubsys(SCPICmdRead):
    """The ``DIAg:NAMe:SUBSYS`` command.

    Description:
        - This query-only command returns the subsystem of the current diagnostic test. This command
          is equivalent to selecting Instrument Diagnostics from the Utilities menu and then
          reviewing the Diagnostic Status. This query-only command can be issued while diagnostics
          are still in progress.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:NAMe:SUBSYS?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:NAMe:SUBSYS?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:NAMe:SUBSYS?
        ```
    """


class DiagNameArea(SCPICmdRead):
    """The ``DIAg:NAMe:AREA`` command.

    Description:
        - This query-only command returns the selected area of the current diagnostic test. There
          are three levels of diagnostic test hierarchy: subsystem, area and test. This command is
          equivalent to selecting Instrument Diagnostics from the Utilities menu and then reviewing
          the Diagnostic Status. This query-only command can be issued while diagnostics are still
          in progress.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:NAMe:AREA?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:NAMe:AREA?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:NAMe:AREA?
        ```
    """


class DiagName(SCPICmdRead):
    """The ``DIAg:NAMe`` command.

    Description:
        - This query-only command returns the names of the subsystem, area, and test of the current
          diagnostic test. This command can be issued while diagnostics are still in progress.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:NAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:NAMe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:NAMe?
        ```

    Properties:
        - ``.area``: The ``DIAg:NAMe:AREA`` command.
        - ``.subsys``: The ``DIAg:NAMe:SUBSYS`` command.
        - ``.test``: The ``DIAg:NAMe:TEST`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._area = DiagNameArea(device, f"{self._cmd_syntax}:AREA")
        self._subsys = DiagNameSubsys(device, f"{self._cmd_syntax}:SUBSYS")
        self._test = DiagNameTest(device, f"{self._cmd_syntax}:TEST")

    @property
    def area(self) -> DiagNameArea:
        """Return the ``DIAg:NAMe:AREA`` command.

        Description:
            - This query-only command returns the selected area of the current diagnostic test.
              There are three levels of diagnostic test hierarchy: subsystem, area and test. This
              command is equivalent to selecting Instrument Diagnostics from the Utilities menu and
              then reviewing the Diagnostic Status. This query-only command can be issued while
              diagnostics are still in progress.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:NAMe:AREA?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:NAMe:AREA?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:NAMe:AREA?
            ```
        """
        return self._area

    @property
    def subsys(self) -> DiagNameSubsys:
        """Return the ``DIAg:NAMe:SUBSYS`` command.

        Description:
            - This query-only command returns the subsystem of the current diagnostic test. This
              command is equivalent to selecting Instrument Diagnostics from the Utilities menu and
              then reviewing the Diagnostic Status. This query-only command can be issued while
              diagnostics are still in progress.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:NAMe:SUBSYS?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:NAMe:SUBSYS?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:NAMe:SUBSYS?
            ```
        """
        return self._subsys

    @property
    def test(self) -> DiagNameTest:
        """Return the ``DIAg:NAMe:TEST`` command.

        Description:
            - This query-only command returns the name of the current diagnostic test. This command
              is equivalent to selecting Instrument Diagnostics from the Utilities menu and then
              reviewing the Diagnostic Status. This query-only command can be issued while
              diagnostics are still in progress.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:NAMe:TEST?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:NAMe:TEST?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:NAMe:TEST?
            ```
        """
        return self._test


class DiagLoops(SCPICmdRead):
    """The ``DIAg:LOOPS`` command.

    Description:
        - This query-only command returns the number of times that the selected diagnostics set was
          completed during the last diagnostic execution. This command is equivalent to selecting
          Instrument Diagnostics from the Utilities menu and then reviewing the Elapsed Loops. This
          query-only command can be issued while diagnostics are still in progress.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:LOOPS?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:LOOPS?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:LOOPS?
        ```
    """


class DiagLevel(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:LEVEL`` command.

    Description:
        - This command sets or queries the selected level of diagnostic test hierarchy. This command
          is equivalent to selecting Instrument Diagnostics from the Utilities menu and then
          reviewing the Diagnostic Status.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:LEVEL?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:LEVEL?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:LEVEL value`` command.

    SCPI Syntax:
        ```
        - DIAg:LEVEL {AREA|SUBSYS|TEST}
        - DIAg:LEVEL?
        ```

    Info:
        - ``AREA`` sets diagnostic testing to the area level.
        - ``SUBSYS`` sets diagnostic testing to the subsystem level.
        - ``TEST`` sets diagnostic testing to the test level.
    """


class DiagItemSubitems(SCPICmdReadWithArguments):
    """The ``DIAg:ITEM:SUBITEMS`` command.

    Description:
        - This query-only command returns the number of subitems associated with the item. This
          command is equivalent to selecting Instrument Diagnostics from the Utilities menu,
          choosing the Subsystem, Area or Test setting and then reviewing the resulting subitems.
          This query-only command can be issued while diagnostics are still in progress.

    Usage:
        - Using the ``.query(argument)`` method will send the ``DIAg:ITEM:SUBITEMS? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``DIAg:ITEM:SUBITEMS? argument`` query and raise an AssertionError if the returned value
          does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:ITEM:SUBITEMS? <NR1>
        ```

    Info:
        - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
          through 15.
    """


class DiagItemResult(SCPICmdReadWithArguments):
    """The ``DIAg:ITEM:RESULT`` command.

    Description:
        - This query-only command returns the result from the last execution of the item. This
          command is equivalent to selecting Instrument Diagnostics from the Utilities menu and then
          reviewing the Diagnostic Status. This query-only command can be issued while diagnostics
          are still in progress.

    Usage:
        - Using the ``.query(argument)`` method will send the ``DIAg:ITEM:RESULT? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``DIAg:ITEM:RESULT? argument``
          query and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:ITEM:RESULT? <NR1>
        ```

    Info:
        - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
          through 15.
    """


class DiagItemName(SCPICmdReadWithArguments):
    """The ``DIAg:ITEM:NAMe`` command.

    Description:
        - This query-only command returns the name of the selected menu item. This command is
          equivalent to selecting Instrument Diagnostics from the Utilities menu and then reviewing
          the Subsystem, Area and Test settings. This query-only command can be issued while
          diagnostics are still in progress.

    Usage:
        - Using the ``.query(argument)`` method will send the ``DIAg:ITEM:NAMe? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``DIAg:ITEM:NAMe? argument``
          query and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:ITEM:NAMe? <NR1>
        ```

    Info:
        - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
          through 15.
    """


class DiagItemFailures(SCPICmdReadWithArguments):
    """The ``DIAg:ITEM:FAILURES`` command.

    Description:
        - This query-only command returns the total number of failures. This command is equivalent
          to selecting Instrument Diagnostics from the Utilities menu and then reviewing the
          Diagnostic Status. This query-only command can be issued while diagnostics are still in
          progress.

    Usage:
        - Using the ``.query(argument)`` method will send the ``DIAg:ITEM:FAILURES? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``DIAg:ITEM:FAILURES? argument`` query and raise an AssertionError if the returned value
          does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:ITEM:FAILURES? <NR1>
        ```

    Info:
        - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
          through 15.
    """


class DiagItem(SCPICmdReadWithArguments):
    """The ``DIAg:ITEM`` command.

    Description:
        - This query-only command returns the diagnostics settings. This command is equivalent to
          selecting Instrument Diagnostics from the Utilities menu, and then reviewing the
          diagnostics settings. This query-only command can be issued while diagnostics are still in
          progress.

    Usage:
        - Using the ``.query(argument)`` method will send the ``DIAg:ITEM? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``DIAg:ITEM? argument`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:ITEM? <NR1>
        ```

    Info:
        - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
          through 15.

    Properties:
        - ``.failures``: The ``DIAg:ITEM:FAILURES`` command.
        - ``.name``: The ``DIAg:ITEM:NAMe`` command.
        - ``.result``: The ``DIAg:ITEM:RESULT`` command.
        - ``.subitems``: The ``DIAg:ITEM:SUBITEMS`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._failures = DiagItemFailures(device, f"{self._cmd_syntax}:FAILURES")
        self._name = DiagItemName(device, f"{self._cmd_syntax}:NAMe")
        self._result = DiagItemResult(device, f"{self._cmd_syntax}:RESULT")
        self._subitems = DiagItemSubitems(device, f"{self._cmd_syntax}:SUBITEMS")

    @property
    def failures(self) -> DiagItemFailures:
        """Return the ``DIAg:ITEM:FAILURES`` command.

        Description:
            - This query-only command returns the total number of failures. This command is
              equivalent to selecting Instrument Diagnostics from the Utilities menu and then
              reviewing the Diagnostic Status. This query-only command can be issued while
              diagnostics are still in progress.

        Usage:
            - Using the ``.query(argument)`` method will send the ``DIAg:ITEM:FAILURES? argument``
              query.
            - Using the ``.verify(argument, value)`` method will send the
              ``DIAg:ITEM:FAILURES? argument`` query and raise an AssertionError if the returned
              value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:ITEM:FAILURES? <NR1>
            ```

        Info:
            - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
              through 15.
        """
        return self._failures

    @property
    def name(self) -> DiagItemName:
        """Return the ``DIAg:ITEM:NAMe`` command.

        Description:
            - This query-only command returns the name of the selected menu item. This command is
              equivalent to selecting Instrument Diagnostics from the Utilities menu and then
              reviewing the Subsystem, Area and Test settings. This query-only command can be issued
              while diagnostics are still in progress.

        Usage:
            - Using the ``.query(argument)`` method will send the ``DIAg:ITEM:NAMe? argument``
              query.
            - Using the ``.verify(argument, value)`` method will send the
              ``DIAg:ITEM:NAMe? argument`` query and raise an AssertionError if the returned value
              does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:ITEM:NAMe? <NR1>
            ```

        Info:
            - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
              through 15.
        """
        return self._name

    @property
    def result(self) -> DiagItemResult:
        """Return the ``DIAg:ITEM:RESULT`` command.

        Description:
            - This query-only command returns the result from the last execution of the item. This
              command is equivalent to selecting Instrument Diagnostics from the Utilities menu and
              then reviewing the Diagnostic Status. This query-only command can be issued while
              diagnostics are still in progress.

        Usage:
            - Using the ``.query(argument)`` method will send the ``DIAg:ITEM:RESULT? argument``
              query.
            - Using the ``.verify(argument, value)`` method will send the
              ``DIAg:ITEM:RESULT? argument`` query and raise an AssertionError if the returned value
              does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:ITEM:RESULT? <NR1>
            ```

        Info:
            - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
              through 15.
        """
        return self._result

    @property
    def subitems(self) -> DiagItemSubitems:
        """Return the ``DIAg:ITEM:SUBITEMS`` command.

        Description:
            - This query-only command returns the number of subitems associated with the item. This
              command is equivalent to selecting Instrument Diagnostics from the Utilities menu,
              choosing the Subsystem, Area or Test setting and then reviewing the resulting
              subitems. This query-only command can be issued while diagnostics are still in
              progress.

        Usage:
            - Using the ``.query(argument)`` method will send the ``DIAg:ITEM:SUBITEMS? argument``
              query.
            - Using the ``.verify(argument, value)`` method will send the
              ``DIAg:ITEM:SUBITEMS? argument`` query and raise an AssertionError if the returned
              value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:ITEM:SUBITEMS? <NR1>
            ```

        Info:
            - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
              through 15.
        """
        return self._subitems


class DiagFailuresClear(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:FAILURES:CLEAR`` command.

    Description:
        - This command sets and queries the clearing of pass/fail information from data structures,
          not the Event Log, at the start of diagnostic tests.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:FAILURES:CLEAR?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:FAILURES:CLEAR?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:FAILURES:CLEAR value`` command.

    SCPI Syntax:
        ```
        - DIAg:FAILURES:CLEAR {ON|OFF|<NR1>}
        - DIAg:FAILURES:CLEAR?
        ```

    Info:
        - ``<NR1> = 0`` turns off the clearing the pass/fail information at the start of tests; any
          other value turns on the clearing of pass/fail information.
        - ``OFF`` does not clear pass/fail information at the start of tests.
        - ``ON`` clears pass/fail information from data structures, not the Event Log, at the start
          of tests.
    """


class DiagFailures(SCPICmdRead):
    """The ``DIAg:FAILURES`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:FAILURES?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:FAILURES?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.clear``: The ``DIAg:FAILURES:CLEAR`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._clear = DiagFailuresClear(device, f"{self._cmd_syntax}:CLEAR")

    @property
    def clear(self) -> DiagFailuresClear:
        """Return the ``DIAg:FAILURES:CLEAR`` command.

        Description:
            - This command sets and queries the clearing of pass/fail information from data
              structures, not the Event Log, at the start of diagnostic tests.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:FAILURES:CLEAR?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:FAILURES:CLEAR?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:FAILURES:CLEAR value``
              command.

        SCPI Syntax:
            ```
            - DIAg:FAILURES:CLEAR {ON|OFF|<NR1>}
            - DIAg:FAILURES:CLEAR?
            ```

        Info:
            - ``<NR1> = 0`` turns off the clearing the pass/fail information at the start of tests;
              any other value turns on the clearing of pass/fail information.
            - ``OFF`` does not clear pass/fail information at the start of tests.
            - ``ON`` clears pass/fail information from data structures, not the Event Log, at the
              start of tests.
        """
        return self._clear


class DiagExecute(SCPICmdWriteNoArguments):
    """The ``DIAg:EXECUTE`` command.

    Description:
        - This command (no query form) starts the execution of the currently selected set of
          diagnostics. This command is equivalent to selecting Instrument Diagnostics from the
          Utilities menu and then pressing Run.

    Usage:
        - Using the ``.write()`` method will send the ``DIAg:EXECUTE`` command.

    SCPI Syntax:
        ```
        - DIAg:EXECUTE
        ```
    """


class DiagControlLoop(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:CONTROL:LOOP`` command.

    Description:
        - This command determines or queries whether the next execution of diagnostics executes once
          or continuously loops on the selected set of diagnostics (assuming the halt control is set
          to off using the ``DIAG:CONTROL:HALT`` command or that the halt control is set to ON but
          no failures occur). This command is equivalent to selecting Instrument Diagnostics from
          the Utilities menu and then enabling Loop Control.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:CONTROL:LOOP?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:CONTROL:LOOP?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:CONTROL:LOOP value`` command.

    SCPI Syntax:
        ```
        - DIAg:CONTROL:LOOP {ON|OFF|<NR1>}
        - DIAg:CONTROL:LOOP?
        ```

    Info:
        - ``<NR1>`` = 1 enables the loop function; any other value disables the loop function.
        - ``ON`` enables the loop function, causing the execution of diagnostics to continuously
          loop.
        - ``OFF`` disables the loop function, causing the instrument to execute the entire set of
          diagnostics once and then halt.
    """


class DiagControlHalt(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:CONTROL:HALT`` command.

    Description:
        - This command determines or queries whether the next execution of diagnostics looping will
          stop on the first diagnostic failure that occurs or will continue to loop on the selected
          set of diagnostic functions. This command is equivalent to selecting Instrument
          Diagnostics from the Utilities menu and then enabling Halt on Fail.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:CONTROL:HALT?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:CONTROL:HALT?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:CONTROL:HALT value`` command.

    SCPI Syntax:
        ```
        - DIAg:CONTROL:HALT {ON|OFF|<NR1>}
        - DIAg:CONTROL:HALT?
        ```

    Info:
        - ``<NR1>`` = 1 enables the halt function; any other value disables the halt function.
        - ``ON`` enables the halt function, causing the execution of diagnostics looping to halt at
          the first diagnostic failure that occurs.
        - ``OFF`` disables the halt looping function, allowing the instrument to continue to loop on
          the entire set of diagnostics, even if diagnostic failure occurs.
    """


class DiagControl(SCPICmdRead):
    """The ``DIAg:CONTROL`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:CONTROL?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:CONTROL?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.halt``: The ``DIAg:CONTROL:HALT`` command.
        - ``.loop``: The ``DIAg:CONTROL:LOOP`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._halt = DiagControlHalt(device, f"{self._cmd_syntax}:HALT")
        self._loop = DiagControlLoop(device, f"{self._cmd_syntax}:LOOP")

    @property
    def halt(self) -> DiagControlHalt:
        """Return the ``DIAg:CONTROL:HALT`` command.

        Description:
            - This command determines or queries whether the next execution of diagnostics looping
              will stop on the first diagnostic failure that occurs or will continue to loop on the
              selected set of diagnostic functions. This command is equivalent to selecting
              Instrument Diagnostics from the Utilities menu and then enabling Halt on Fail.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:CONTROL:HALT?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:CONTROL:HALT?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:CONTROL:HALT value`` command.

        SCPI Syntax:
            ```
            - DIAg:CONTROL:HALT {ON|OFF|<NR1>}
            - DIAg:CONTROL:HALT?
            ```

        Info:
            - ``<NR1>`` = 1 enables the halt function; any other value disables the halt function.
            - ``ON`` enables the halt function, causing the execution of diagnostics looping to halt
              at the first diagnostic failure that occurs.
            - ``OFF`` disables the halt looping function, allowing the instrument to continue to
              loop on the entire set of diagnostics, even if diagnostic failure occurs.
        """
        return self._halt

    @property
    def loop(self) -> DiagControlLoop:
        """Return the ``DIAg:CONTROL:LOOP`` command.

        Description:
            - This command determines or queries whether the next execution of diagnostics executes
              once or continuously loops on the selected set of diagnostics (assuming the halt
              control is set to off using the ``DIAG:CONTROL:HALT`` command or that the halt control
              is set to ON but no failures occur). This command is equivalent to selecting
              Instrument Diagnostics from the Utilities menu and then enabling Loop Control.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:CONTROL:LOOP?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:CONTROL:LOOP?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:CONTROL:LOOP value`` command.

        SCPI Syntax:
            ```
            - DIAg:CONTROL:LOOP {ON|OFF|<NR1>}
            - DIAg:CONTROL:LOOP?
            ```

        Info:
            - ``<NR1>`` = 1 enables the loop function; any other value disables the loop function.
            - ``ON`` enables the loop function, causing the execution of diagnostics to continuously
              loop.
            - ``OFF`` disables the loop function, causing the instrument to execute the entire set
              of diagnostics once and then halt.
        """
        return self._loop


#  pylint: disable=too-many-instance-attributes
class Diag(SCPICmdRead):
    """The ``DIAg`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.control``: The ``DIAg:CONTROL`` command tree.
        - ``.execute``: The ``DIAg:EXECUTE`` command.
        - ``.failures``: The ``DIAg:FAILURES`` command tree.
        - ``.item``: The ``DIAg:ITEM`` command.
        - ``.level``: The ``DIAg:LEVEL`` command.
        - ``.loops``: The ``DIAg:LOOPS`` command.
        - ``.name``: The ``DIAg:NAMe`` command.
        - ``.numitems``: The ``DIAg:NUMITEMS`` command.
        - ``.results``: The ``DIAg:RESults`` command.
        - ``.select``: The ``DIAg:SELect`` command tree.
        - ``.state``: The ``DIAg:STATE`` command.
        - ``.stop``: The ``DIAg:STOP`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "DIAg") -> None:
        super().__init__(device, cmd_syntax)
        self._control = DiagControl(device, f"{self._cmd_syntax}:CONTROL")
        self._execute = DiagExecute(device, f"{self._cmd_syntax}:EXECUTE")
        self._failures = DiagFailures(device, f"{self._cmd_syntax}:FAILURES")
        self._item = DiagItem(device, f"{self._cmd_syntax}:ITEM")
        self._level = DiagLevel(device, f"{self._cmd_syntax}:LEVEL")
        self._loops = DiagLoops(device, f"{self._cmd_syntax}:LOOPS")
        self._name = DiagName(device, f"{self._cmd_syntax}:NAMe")
        self._numitems = DiagNumitems(device, f"{self._cmd_syntax}:NUMITEMS")
        self._results = DiagResults(device, f"{self._cmd_syntax}:RESults")
        self._select = DiagSelect(device, f"{self._cmd_syntax}:SELect")
        self._state = DiagState(device, f"{self._cmd_syntax}:STATE")
        self._stop = DiagStop(device, f"{self._cmd_syntax}:STOP")

    @property
    def control(self) -> DiagControl:
        """Return the ``DIAg:CONTROL`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:CONTROL?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:CONTROL?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.halt``: The ``DIAg:CONTROL:HALT`` command.
            - ``.loop``: The ``DIAg:CONTROL:LOOP`` command.
        """
        return self._control

    @property
    def execute(self) -> DiagExecute:
        """Return the ``DIAg:EXECUTE`` command.

        Description:
            - This command (no query form) starts the execution of the currently selected set of
              diagnostics. This command is equivalent to selecting Instrument Diagnostics from the
              Utilities menu and then pressing Run.

        Usage:
            - Using the ``.write()`` method will send the ``DIAg:EXECUTE`` command.

        SCPI Syntax:
            ```
            - DIAg:EXECUTE
            ```
        """
        return self._execute

    @property
    def failures(self) -> DiagFailures:
        """Return the ``DIAg:FAILURES`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:FAILURES?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:FAILURES?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.clear``: The ``DIAg:FAILURES:CLEAR`` command.
        """
        return self._failures

    @property
    def item(self) -> DiagItem:
        """Return the ``DIAg:ITEM`` command.

        Description:
            - This query-only command returns the diagnostics settings. This command is equivalent
              to selecting Instrument Diagnostics from the Utilities menu, and then reviewing the
              diagnostics settings. This query-only command can be issued while diagnostics are
              still in progress.

        Usage:
            - Using the ``.query(argument)`` method will send the ``DIAg:ITEM? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the ``DIAg:ITEM? argument``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:ITEM? <NR1>
            ```

        Info:
            - ``<NR1>`` sets the index item about which data will be returned, which ranges from 0
              through 15.

        Sub-properties:
            - ``.failures``: The ``DIAg:ITEM:FAILURES`` command.
            - ``.name``: The ``DIAg:ITEM:NAMe`` command.
            - ``.result``: The ``DIAg:ITEM:RESULT`` command.
            - ``.subitems``: The ``DIAg:ITEM:SUBITEMS`` command.
        """
        return self._item

    @property
    def level(self) -> DiagLevel:
        """Return the ``DIAg:LEVEL`` command.

        Description:
            - This command sets or queries the selected level of diagnostic test hierarchy. This
              command is equivalent to selecting Instrument Diagnostics from the Utilities menu and
              then reviewing the Diagnostic Status.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:LEVEL?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:LEVEL?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:LEVEL value`` command.

        SCPI Syntax:
            ```
            - DIAg:LEVEL {AREA|SUBSYS|TEST}
            - DIAg:LEVEL?
            ```

        Info:
            - ``AREA`` sets diagnostic testing to the area level.
            - ``SUBSYS`` sets diagnostic testing to the subsystem level.
            - ``TEST`` sets diagnostic testing to the test level.
        """
        return self._level

    @property
    def loops(self) -> DiagLoops:
        """Return the ``DIAg:LOOPS`` command.

        Description:
            - This query-only command returns the number of times that the selected diagnostics set
              was completed during the last diagnostic execution. This command is equivalent to
              selecting Instrument Diagnostics from the Utilities menu and then reviewing the
              Elapsed Loops. This query-only command can be issued while diagnostics are still in
              progress.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:LOOPS?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:LOOPS?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:LOOPS?
            ```
        """
        return self._loops

    @property
    def name(self) -> DiagName:
        """Return the ``DIAg:NAMe`` command.

        Description:
            - This query-only command returns the names of the subsystem, area, and test of the
              current diagnostic test. This command can be issued while diagnostics are still in
              progress.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:NAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:NAMe?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:NAMe?
            ```

        Sub-properties:
            - ``.area``: The ``DIAg:NAMe:AREA`` command.
            - ``.subsys``: The ``DIAg:NAMe:SUBSYS`` command.
            - ``.test``: The ``DIAg:NAMe:TEST`` command.
        """
        return self._name

    @property
    def numitems(self) -> DiagNumitems:
        """Return the ``DIAg:NUMITEMS`` command.

        Description:
            - This query-only command returns the number of items on the currently selected level of
              test hierarchy, which ranges from 1 through 15. This command is equivalent to
              selecting Instrument Diagnostics from the Utilities menu and then reviewing the
              Diagnostic Status. This query-only command can be issued while diagnostics are still
              in progress.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:NUMITEMS?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:NUMITEMS?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:NUMITEMS?
            ```
        """
        return self._numitems

    @property
    def results(self) -> DiagResults:
        """Return the ``DIAg:RESults`` command.

        Description:
            - This query-only command returns an abbreviated status about the results of the last
              diagnostic (or test) execution. For a more explanatory status message, use the
              ``DIAg:RESults:VERBose?`` query. This command is equivalent to selecting Instrument
              Diagnostics from the Utilities menu and then reviewing the Diagnostic Status. This
              query-only command can be issued while diagnostics are still in progress.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:RESults?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:RESults?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:RESults?
            ```

        Sub-properties:
            - ``.verbose``: The ``DIAg:RESults:VERBose`` command.
        """
        return self._results

    @property
    def select(self) -> DiagSelect:
        """Return the ``DIAg:SELect`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:SELect?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:SELect?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.all``: The ``DIAg:SELect:ALL`` command.
            - ``.area``: The ``DIAg:SELect:AREA`` command.
            - ``.last``: The ``DIAg:SELect:LAST`` command.
            - ``.subsys``: The ``DIAg:SELect:SUBSYS`` command.
            - ``.test``: The ``DIAg:SELect:TEST`` command.
        """
        return self._select

    @property
    def state(self) -> DiagState:
        """Return the ``DIAg:STATE`` command.

        Description:
            - This command changes the instrument operating state. Depending on the argument,
              diagnostics capabilities are either turned on or off. This command is equivalent to
              opening the ``DIAg:STATE`` dialog (ON) or closing it (OFF).

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:STATE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:STATE value`` command.

        SCPI Syntax:
            ```
            - DIAg:STATE {<NR1>|EXECUTE|ON|OFF}
            - DIAg:STATE?
            ```

        Info:
            - ``<NR1>`` = 0 disables diagnostics capabilities and returns the instrument to a normal
              operating state; any other value enables diagnostics.
            - ``EXECUTE`` starts execution of the diagnostics.
            - ``ON`` puts the instrument into the state in which diagnostics can be run. This
              argument is thrown automatically if the ``DIAG:EXECUTE`` command is executed.
            - ``OFF`` disables diagnostics capabilities and returns the instrument to a normal
              operating state.
        """
        return self._state

    @property
    def stop(self) -> DiagStop:
        """Return the ``DIAg:STOP`` command.

        Description:
            - This command (no query form) causes diagnostics (or test) execution to terminate at
              the end of the next low-level test. This command is equivalent to selecting Instrument
              Diagnostics from the Utilities menu and then clicking Abort.

        Usage:
            - Using the ``.write()`` method will send the ``DIAg:STOP`` command.

        SCPI Syntax:
            ```
            - DIAg:STOP
            ```
        """
        return self._stop



"""A mixin class that contains common methods for checking the PI device for SYSTEM:ERROR."""

from abc import ABC
from typing import List, Tuple

from tm_devices.driver_mixins.device_control._abstract_device_visa_write_query_control import (
    _AbstractDeviceVISAWriteQueryControl,  # pyright: ignore[reportPrivateUsage]
)


class CommonPISystemErrorCheckMixin(_AbstractDeviceVISAWriteQueryControl, ABC):
    """A mixin class that contains common methods for checking the PI device for SYSTEM:ERROR.

    !!! important
        Any class that inherits this mixin must also inherit the
        [`PIControl`][tm_devices.driver_mixins.device_control.PIControl] mixin in order
        to have access to the methods required by this class.
    """

    @property
    def _no_error_string(self) -> str:
        """A string containing the expected error message when no error is present."""
        return '0,"No error"'

    def _get_errors(self) -> Tuple[int, Tuple[str, ...]]:
        """Get the current errors from the device.

        !!! note
            This method will clear out the error queue after reading the current errors.

        Returns:
            A tuple containing the current error code alongside a tuple of the current error
                messages.
        """
        result = int(self.query("*ESR?").strip())

        # return the errors if any
        returned_errors: List[str] = []
        error = ""
        while error != self._no_error_string:
            error = self.query("SYSTEM:ERROR?")
            returned_errors.append(error)

        return result, tuple(returned_errors)



"""The autoset commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AUTOSet EXECute
    - AUTOSet:ACQuisition:ENAble {ON|OFF|1|0}
    - AUTOSet:ACQuisition:ENAble?
    - AUTOSet:ENAble {ON|OFF|1|0}
    - AUTOSet:ENAble?
    - AUTOSet:HORizontal:ENAble {ON|OFF|1|0}
    - AUTOSet:HORizontal:ENAble?
    - AUTOSet:TRIGger:ENAble {ON|OFF|1|0}
    - AUTOSet:TRIGger:ENAble?
    - AUTOSet:VERTical:ENAble {ON|OFF|1|0}
    - AUTOSet:VERTical:ENAble?
    - AUTOSet:VERTical:OPTIMize {RESOlution|VISIBility}
    - AUTOSet:VERTical:OPTIMize?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AutosetVerticalOptimize(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet:VERTical:OPTIMize`` command.

    Description:
        - This command sets or queries which vertical settings Autoset will optimize when the
          display mode is set to Overlay mode (all waveforms are in one common graticule in the
          Waveform View).

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:VERTical:OPTIMize?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:VERTical:OPTIMize?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSet:VERTical:OPTIMize value``
          command.

    SCPI Syntax:
        ```
        - AUTOSet:VERTical:OPTIMize {RESOlution|VISIBility}
        - AUTOSet:VERTical:OPTIMize?
        ```

    Info:
        - ``RESOlution`` uses as much of the ADC's (Analog to Digital Converter) range as possible
          to provide the best vertical resolution and measurement accuracy, but waveforms will
          overlap each other.
        - ``VISIBility`` vertically scales and positions waveforms so they are visually separated
          from each other at the expense of vertical resolution and measurement accuracy.
    """


class AutosetVerticalEnable(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet:VERTical:ENAble`` command.

    Description:
        - This command sets or queries Autoset's adjustment of vertical settings. Settings affected
          may include, but not be limited to, vertical scale, vertical position, and vertical
          offset.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:VERTical:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:VERTical:ENAble?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSet:VERTical:ENAble value``
          command.

    SCPI Syntax:
        ```
        - AUTOSet:VERTical:ENAble {ON|OFF|1|0}
        - AUTOSet:VERTical:ENAble?
        ```

    Info:
        - ``ON`` or 1 enables Autoset to change vertical settings.
        - ``OFF`` or 0 disables Autoset from changing vertical settings.
    """


class AutosetVertical(SCPICmdRead):
    """The ``AUTOSet:VERTical`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:VERTical?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:VERTical?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.enable``: The ``AUTOSet:VERTical:ENAble`` command.
        - ``.optimize``: The ``AUTOSet:VERTical:OPTIMize`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._enable = AutosetVerticalEnable(device, f"{self._cmd_syntax}:ENAble")
        self._optimize = AutosetVerticalOptimize(device, f"{self._cmd_syntax}:OPTIMize")

    @property
    def enable(self) -> AutosetVerticalEnable:
        """Return the ``AUTOSet:VERTical:ENAble`` command.

        Description:
            - This command sets or queries Autoset's adjustment of vertical settings. Settings
              affected may include, but not be limited to, vertical scale, vertical position, and
              vertical offset.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:VERTical:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:VERTical:ENAble?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUTOSet:VERTical:ENAble value``
              command.

        SCPI Syntax:
            ```
            - AUTOSet:VERTical:ENAble {ON|OFF|1|0}
            - AUTOSet:VERTical:ENAble?
            ```

        Info:
            - ``ON`` or 1 enables Autoset to change vertical settings.
            - ``OFF`` or 0 disables Autoset from changing vertical settings.
        """
        return self._enable

    @property
    def optimize(self) -> AutosetVerticalOptimize:
        """Return the ``AUTOSet:VERTical:OPTIMize`` command.

        Description:
            - This command sets or queries which vertical settings Autoset will optimize when the
              display mode is set to Overlay mode (all waveforms are in one common graticule in the
              Waveform View).

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:VERTical:OPTIMize?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:VERTical:OPTIMize?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUTOSet:VERTical:OPTIMize value``
              command.

        SCPI Syntax:
            ```
            - AUTOSet:VERTical:OPTIMize {RESOlution|VISIBility}
            - AUTOSet:VERTical:OPTIMize?
            ```

        Info:
            - ``RESOlution`` uses as much of the ADC's (Analog to Digital Converter) range as
              possible to provide the best vertical resolution and measurement accuracy, but
              waveforms will overlap each other.
            - ``VISIBility`` vertically scales and positions waveforms so they are visually
              separated from each other at the expense of vertical resolution and measurement
              accuracy.
        """
        return self._optimize


class AutosetTriggerEnable(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet:TRIGger:ENAble`` command.

    Description:
        - This command sets or queries Autoset's adjustment of trigger settings. Settings affected
          may include, but not be limited to, trigger level, trigger source, and trigger coupling.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:TRIGger:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:TRIGger:ENAble?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSet:TRIGger:ENAble value`` command.

    SCPI Syntax:
        ```
        - AUTOSet:TRIGger:ENAble {ON|OFF|1|0}
        - AUTOSet:TRIGger:ENAble?
        ```

    Info:
        - ``ON`` or 1 enables Autoset to change trigger settings.
        - ``OFF`` or 0 disables Autoset from changing trigger settings.
    """


class AutosetTrigger(SCPICmdRead):
    """The ``AUTOSet:TRIGger`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:TRIGger?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:TRIGger?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.enable``: The ``AUTOSet:TRIGger:ENAble`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._enable = AutosetTriggerEnable(device, f"{self._cmd_syntax}:ENAble")

    @property
    def enable(self) -> AutosetTriggerEnable:
        """Return the ``AUTOSet:TRIGger:ENAble`` command.

        Description:
            - This command sets or queries Autoset's adjustment of trigger settings. Settings
              affected may include, but not be limited to, trigger level, trigger source, and
              trigger coupling.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:TRIGger:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:TRIGger:ENAble?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUTOSet:TRIGger:ENAble value``
              command.

        SCPI Syntax:
            ```
            - AUTOSet:TRIGger:ENAble {ON|OFF|1|0}
            - AUTOSet:TRIGger:ENAble?
            ```

        Info:
            - ``ON`` or 1 enables Autoset to change trigger settings.
            - ``OFF`` or 0 disables Autoset from changing trigger settings.
        """
        return self._enable


class AutosetHorizontalEnable(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet:HORizontal:ENAble`` command.

    Description:
        - This command sets or queries Autoset's adjustment of horizontal settings. Settings
          affected may include, but not be limited to, horizontal scale, horizontal position, and
          horizontal delay mode.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:HORizontal:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:HORizontal:ENAble?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSet:HORizontal:ENAble value``
          command.

    SCPI Syntax:
        ```
        - AUTOSet:HORizontal:ENAble {ON|OFF|1|0}
        - AUTOSet:HORizontal:ENAble?
        ```

    Info:
        - ``ON`` or 1 enables Autoset to change horizontal settings.
        - ``OFF`` or 0 disables Autoset from changing horizontal settings.
    """


class AutosetHorizontal(SCPICmdRead):
    """The ``AUTOSet:HORizontal`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:HORizontal?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:HORizontal?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.enable``: The ``AUTOSet:HORizontal:ENAble`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._enable = AutosetHorizontalEnable(device, f"{self._cmd_syntax}:ENAble")

    @property
    def enable(self) -> AutosetHorizontalEnable:
        """Return the ``AUTOSet:HORizontal:ENAble`` command.

        Description:
            - This command sets or queries Autoset's adjustment of horizontal settings. Settings
              affected may include, but not be limited to, horizontal scale, horizontal position,
              and horizontal delay mode.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:HORizontal:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:HORizontal:ENAble?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUTOSet:HORizontal:ENAble value``
              command.

        SCPI Syntax:
            ```
            - AUTOSet:HORizontal:ENAble {ON|OFF|1|0}
            - AUTOSet:HORizontal:ENAble?
            ```

        Info:
            - ``ON`` or 1 enables Autoset to change horizontal settings.
            - ``OFF`` or 0 disables Autoset from changing horizontal settings.
        """
        return self._enable


class AutosetEnable(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet:ENAble`` command.

    Description:
        - This command sets or queries the Autoset enable/disable feature. This is useful for
          classroom purposes where the instructor wants the students to achieve the desired
          instrument settings without the benefit of the Autoset feature.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:ENAble?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSet:ENAble value`` command.

    SCPI Syntax:
        ```
        - AUTOSet:ENAble {ON|OFF|1|0}
        - AUTOSet:ENAble?
        ```

    Info:
        - ``ON`` or 1 enables Autoset.
        - ``OFF`` or 0 disables Autoset.
    """


class AutosetAcquisitionEnable(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet:ACQuisition:ENAble`` command.

    Description:
        - This command sets or queries the Autoset acquisition setting adjustment. Settings affected
          may include, but not be limited to, acquisition mode, and FastAcq mode.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:ACQuisition:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:ACQuisition:ENAble?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSet:ACQuisition:ENAble value``
          command.

    SCPI Syntax:
        ```
        - AUTOSet:ACQuisition:ENAble {ON|OFF|1|0}
        - AUTOSet:ACQuisition:ENAble?
        ```

    Info:
        - ``ON`` or 1 enables Autoset to change acquisition settings.
        - ``OFF`` or 0 disables Autoset from changing acquisition settings.
    """


class AutosetAcquisition(SCPICmdRead):
    """The ``AUTOSet:ACQuisition`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:ACQuisition?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:ACQuisition?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.enable``: The ``AUTOSet:ACQuisition:ENAble`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._enable = AutosetAcquisitionEnable(device, f"{self._cmd_syntax}:ENAble")

    @property
    def enable(self) -> AutosetAcquisitionEnable:
        """Return the ``AUTOSet:ACQuisition:ENAble`` command.

        Description:
            - This command sets or queries the Autoset acquisition setting adjustment. Settings
              affected may include, but not be limited to, acquisition mode, and FastAcq mode.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:ACQuisition:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:ACQuisition:ENAble?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUTOSet:ACQuisition:ENAble value``
              command.

        SCPI Syntax:
            ```
            - AUTOSet:ACQuisition:ENAble {ON|OFF|1|0}
            - AUTOSet:ACQuisition:ENAble?
            ```

        Info:
            - ``ON`` or 1 enables Autoset to change acquisition settings.
            - ``OFF`` or 0 disables Autoset from changing acquisition settings.
        """
        return self._enable


class Autoset(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet`` command.

    Description:
        - This command (no query format) sets the vertical, horizontal, and trigger controls of the
          instrument to automatically acquire and display the selected waveform.

    Usage:
        - Using the ``.write(value)`` method will send the ``AUTOSet value`` command.

    SCPI Syntax:
        ```
        - AUTOSet EXECute
        ```

    Info:
        - ``EXECute`` autosets the displayed waveform; this is equivalent to pressing the front
          panel Autoset button.

    Properties:
        - ``.acquisition``: The ``AUTOSet:ACQuisition`` command tree.
        - ``.enable``: The ``AUTOSet:ENAble`` command.
        - ``.horizontal``: The ``AUTOSet:HORizontal`` command tree.
        - ``.trigger``: The ``AUTOSet:TRIGger`` command tree.
        - ``.vertical``: The ``AUTOSet:VERTical`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "AUTOSet") -> None:
        super().__init__(device, cmd_syntax)
        self._acquisition = AutosetAcquisition(device, f"{self._cmd_syntax}:ACQuisition")
        self._enable = AutosetEnable(device, f"{self._cmd_syntax}:ENAble")
        self._horizontal = AutosetHorizontal(device, f"{self._cmd_syntax}:HORizontal")
        self._trigger = AutosetTrigger(device, f"{self._cmd_syntax}:TRIGger")
        self._vertical = AutosetVertical(device, f"{self._cmd_syntax}:VERTical")

    @property
    def acquisition(self) -> AutosetAcquisition:
        """Return the ``AUTOSet:ACQuisition`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:ACQuisition?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:ACQuisition?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.enable``: The ``AUTOSet:ACQuisition:ENAble`` command.
        """
        return self._acquisition

    @property
    def enable(self) -> AutosetEnable:
        """Return the ``AUTOSet:ENAble`` command.

        Description:
            - This command sets or queries the Autoset enable/disable feature. This is useful for
              classroom purposes where the instructor wants the students to achieve the desired
              instrument settings without the benefit of the Autoset feature.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:ENAble?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUTOSet:ENAble value`` command.

        SCPI Syntax:
            ```
            - AUTOSet:ENAble {ON|OFF|1|0}
            - AUTOSet:ENAble?
            ```

        Info:
            - ``ON`` or 1 enables Autoset.
            - ``OFF`` or 0 disables Autoset.
        """
        return self._enable

    @property
    def horizontal(self) -> AutosetHorizontal:
        """Return the ``AUTOSet:HORizontal`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:HORizontal?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:HORizontal?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.enable``: The ``AUTOSet:HORizontal:ENAble`` command.
        """
        return self._horizontal

    @property
    def trigger(self) -> AutosetTrigger:
        """Return the ``AUTOSet:TRIGger`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:TRIGger?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:TRIGger?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.enable``: The ``AUTOSet:TRIGger:ENAble`` command.
        """
        return self._trigger

    @property
    def vertical(self) -> AutosetVertical:
        """Return the ``AUTOSet:VERTical`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:VERTical?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:VERTical?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.enable``: The ``AUTOSet:VERTical:ENAble`` command.
            - ``.optimize``: The ``AUTOSet:VERTical:OPTIMize`` command.
        """
        return self._vertical



"""The pause commands module.

These commands are used in the following models:
DPO4K, DPO4KB, DPO7AX, LPD6, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO4, MSO4B, MSO4K, MSO4KB,
MSO5, MSO5B, MSO5LP, MSO6, MSO6B, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - PAUSe <NR3>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Pause(SCPICmdWrite):
    """The ``PAUSe`` command.

    Description:
        - This command causes the interface to pause the specified number of seconds before
          processing any other commands.

    Usage:
        - Using the ``.write(value)`` method will send the ``PAUSe value`` command.

    SCPI Syntax:
        ```
        - PAUSe <NR3>
        ```

    Info:
        - ``<NR3>`` is the specified number of seconds the interface is to pause before processing
          any other commands. The pause time is specified as a floating point value in seconds and
          must be > 0.0 and ≥1800.0.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "PAUSe") -> None:
        super().__init__(device, cmd_syntax)



# pylint: disable=line-too-long
"""The afg commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AFG:AMPLitude <NR3>
    - AFG:AMPLitude?
    - AFG:ARBitrary:SOUrce <QString>
    - AFG:ARBitrary:SOUrce?
    - AFG:BURSt:CCOUnt <NR1>
    - AFG:BURSt:CCOUnt?
    - AFG:BURSt:TRIGger
    - AFG:FREQuency <NR3>
    - AFG:FREQuency?
    - AFG:FUNCtion {SINE|SQUare|PULSe|RAMP|NOISe|DC|SINC|GAUSsian|LORENtz|ERISe|EDECAy|HAVERSINe|CARDIac|ARBitrary}
    - AFG:FUNCtion?
    - AFG:HIGHLevel <NR3>
    - AFG:HIGHLevel?
    - AFG:LOWLevel <NR3>
    - AFG:LOWLevel?
    - AFG:NOISEAdd:PERCent <NR3>
    - AFG:NOISEAdd:PERCent?
    - AFG:NOISEAdd:STATE {ON|OFF|1|0}
    - AFG:NOISEAdd:STATE?
    - AFG:OFFSet <NR3>
    - AFG:OFFSet?
    - AFG:OUTPut:LOAd:IMPEDance {FIFty|HIGHZ}
    - AFG:OUTPut:LOAd:IMPEDance?
    - AFG:OUTPut:MODe {OFF|CONTinuous|BURSt}
    - AFG:OUTPut:MODe?
    - AFG:OUTPut:STATE {ON|OFF|1|0}
    - AFG:OUTPut:STATE?
    - AFG:PERIod <NR3>
    - AFG:PERIod?
    - AFG:PULse:WIDth <NR3>
    - AFG:PULse:WIDth?
    - AFG:RAMP:SYMmetry <NR3>
    - AFG:RAMP:SYMmetry?
    - AFG:SQUare:DUty <NR3>
    - AFG:SQUare:DUty?
    ```
"""  # noqa: E501

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AfgSquareDuty(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:SQUare:DUty`` command.

    Description:
        - Sets (or queries) the AFG duty cycle in percent. The minimum is 10.0%, maximum is 90.0%
          and increment is 0.10%.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:SQUare:DUty?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:SQUare:DUty?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:SQUare:DUty value`` command.

    SCPI Syntax:
        ```
        - AFG:SQUare:DUty <NR3>
        - AFG:SQUare:DUty?
        ```

    Info:
        - ``<NR3>`` is a floating point number that represents the AFG duty cycle as a percentage.
    """


class AfgSquare(SCPICmdRead):
    """The ``AFG:SQUare`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:SQUare?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:SQUare?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.duty``: The ``AFG:SQUare:DUty`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._duty = AfgSquareDuty(device, f"{self._cmd_syntax}:DUty")

    @property
    def duty(self) -> AfgSquareDuty:
        """Return the ``AFG:SQUare:DUty`` command.

        Description:
            - Sets (or queries) the AFG duty cycle in percent. The minimum is 10.0%, maximum is
              90.0% and increment is 0.10%.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:SQUare:DUty?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:SQUare:DUty?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:SQUare:DUty value`` command.

        SCPI Syntax:
            ```
            - AFG:SQUare:DUty <NR3>
            - AFG:SQUare:DUty?
            ```

        Info:
            - ``<NR3>`` is a floating point number that represents the AFG duty cycle as a
              percentage.
        """
        return self._duty


class AfgRampSymmetry(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:RAMP:SYMmetry`` command.

    Description:
        - Sets (or queries) the AFG ramp symmetry in percent. Minimum is 0.0%, maximum is 100.0% and
          increment is 0.10%.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:RAMP:SYMmetry?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:RAMP:SYMmetry?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:RAMP:SYMmetry value`` command.

    SCPI Syntax:
        ```
        - AFG:RAMP:SYMmetry <NR3>
        - AFG:RAMP:SYMmetry?
        ```

    Info:
        - ``<NR3>`` is a floating point number that represents the AFG ramp symmetry as a
          percentage.
    """


class AfgRamp(SCPICmdRead):
    """The ``AFG:RAMP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:RAMP?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:RAMP?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.symmetry``: The ``AFG:RAMP:SYMmetry`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._symmetry = AfgRampSymmetry(device, f"{self._cmd_syntax}:SYMmetry")

    @property
    def symmetry(self) -> AfgRampSymmetry:
        """Return the ``AFG:RAMP:SYMmetry`` command.

        Description:
            - Sets (or queries) the AFG ramp symmetry in percent. Minimum is 0.0%, maximum is 100.0%
              and increment is 0.10%.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:RAMP:SYMmetry?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:RAMP:SYMmetry?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:RAMP:SYMmetry value`` command.

        SCPI Syntax:
            ```
            - AFG:RAMP:SYMmetry <NR3>
            - AFG:RAMP:SYMmetry?
            ```

        Info:
            - ``<NR3>`` is a floating point number that represents the AFG ramp symmetry as a
              percentage.
        """
        return self._symmetry


class AfgPulseWidth(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:PULse:WIDth`` command.

    Description:
        - Sets (or queries) the AFG pulse width, in seconds.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:PULse:WIDth?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:PULse:WIDth?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:PULse:WIDth value`` command.

    SCPI Syntax:
        ```
        - AFG:PULse:WIDth <NR3>
        - AFG:PULse:WIDth?
        ```

    Info:
        - ``<NR3>`` is the floating point number that represents the pulse width, in seconds.
    """


class AfgPulse(SCPICmdRead):
    """The ``AFG:PULse`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:PULse?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:PULse?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.width``: The ``AFG:PULse:WIDth`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._width = AfgPulseWidth(device, f"{self._cmd_syntax}:WIDth")

    @property
    def width(self) -> AfgPulseWidth:
        """Return the ``AFG:PULse:WIDth`` command.

        Description:
            - Sets (or queries) the AFG pulse width, in seconds.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:PULse:WIDth?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:PULse:WIDth?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:PULse:WIDth value`` command.

        SCPI Syntax:
            ```
            - AFG:PULse:WIDth <NR3>
            - AFG:PULse:WIDth?
            ```

        Info:
            - ``<NR3>`` is the floating point number that represents the pulse width, in seconds.
        """
        return self._width


class AfgPeriod(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:PERIod`` command.

    Description:
        - Sets (or queries) the period of the AFG waveform, in seconds.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:PERIod?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:PERIod?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:PERIod value`` command.

    SCPI Syntax:
        ```
        - AFG:PERIod <NR3>
        - AFG:PERIod?
        ```

    Info:
        - ``NR3`` is the floating point number that represents the AFG period value, in seconds.
    """


class AfgOutputState(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:OUTPut:STATE`` command.

    Description:
        - Sets (or queries) the AFG output state.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:OUTPut:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:OUTPut:STATE value`` command.

    SCPI Syntax:
        ```
        - AFG:OUTPut:STATE {ON|OFF|1|0}
        - AFG:OUTPut:STATE?
        ```

    Info:
        - ``1`` or ON turns on the AFG output state.
        - ``0`` or OFF turns it off.
    """


class AfgOutputMode(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:OUTPut:MODe`` command.

    Description:
        - This command sets or returns the AFG output mode.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:OUTPut:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:OUTPut:MODe value`` command.

    SCPI Syntax:
        ```
        - AFG:OUTPut:MODe {OFF|CONTinuous|BURSt}
        - AFG:OUTPut:MODe?
        ```

    Info:
        - ``OFF`` turns off the AFG output mode.
        - ``CONTinuous`` turns the AFG output mode to continuous.
        - ``BURSt`` turns the AFG output mode to burst.
    """


class AfgOutputLoadImpedance(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:OUTPut:LOAd:IMPEDance`` command.

    Description:
        - Sets (or queries) the AFG output load impedance.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:OUTPut:LOAd:IMPEDance?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:LOAd:IMPEDance?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:OUTPut:LOAd:IMPEDance value``
          command.

    SCPI Syntax:
        ```
        - AFG:OUTPut:LOAd:IMPEDance {FIFty|HIGHZ}
        - AFG:OUTPut:LOAd:IMPEDance?
        ```

    Info:
        - ``FIFty`` sets the output load impedance to 50 Ohms.
        - ``HIGHZ`` sets the output load impedance to the high-impedance state.
    """


class AfgOutputLoad(SCPICmdRead):
    """The ``AFG:OUTPut:LOAd`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:OUTPut:LOAd?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:LOAd?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.impedance``: The ``AFG:OUTPut:LOAd:IMPEDance`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._impedance = AfgOutputLoadImpedance(device, f"{self._cmd_syntax}:IMPEDance")

    @property
    def impedance(self) -> AfgOutputLoadImpedance:
        """Return the ``AFG:OUTPut:LOAd:IMPEDance`` command.

        Description:
            - Sets (or queries) the AFG output load impedance.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:OUTPut:LOAd:IMPEDance?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:LOAd:IMPEDance?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:OUTPut:LOAd:IMPEDance value``
              command.

        SCPI Syntax:
            ```
            - AFG:OUTPut:LOAd:IMPEDance {FIFty|HIGHZ}
            - AFG:OUTPut:LOAd:IMPEDance?
            ```

        Info:
            - ``FIFty`` sets the output load impedance to 50 Ohms.
            - ``HIGHZ`` sets the output load impedance to the high-impedance state.
        """
        return self._impedance


class AfgOutput(SCPICmdRead):
    """The ``AFG:OUTPut`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:OUTPut?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OUTPut?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.load``: The ``AFG:OUTPut:LOAd`` command tree.
        - ``.mode``: The ``AFG:OUTPut:MODe`` command.
        - ``.state``: The ``AFG:OUTPut:STATE`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._load = AfgOutputLoad(device, f"{self._cmd_syntax}:LOAd")
        self._mode = AfgOutputMode(device, f"{self._cmd_syntax}:MODe")
        self._state = AfgOutputState(device, f"{self._cmd_syntax}:STATE")

    @property
    def load(self) -> AfgOutputLoad:
        """Return the ``AFG:OUTPut:LOAd`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:OUTPut:LOAd?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:LOAd?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.impedance``: The ``AFG:OUTPut:LOAd:IMPEDance`` command.
        """
        return self._load

    @property
    def mode(self) -> AfgOutputMode:
        """Return the ``AFG:OUTPut:MODe`` command.

        Description:
            - This command sets or returns the AFG output mode.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:OUTPut:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:MODe?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:OUTPut:MODe value`` command.

        SCPI Syntax:
            ```
            - AFG:OUTPut:MODe {OFF|CONTinuous|BURSt}
            - AFG:OUTPut:MODe?
            ```

        Info:
            - ``OFF`` turns off the AFG output mode.
            - ``CONTinuous`` turns the AFG output mode to continuous.
            - ``BURSt`` turns the AFG output mode to burst.
        """
        return self._mode

    @property
    def state(self) -> AfgOutputState:
        """Return the ``AFG:OUTPut:STATE`` command.

        Description:
            - Sets (or queries) the AFG output state.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:OUTPut:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut:STATE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:OUTPut:STATE value`` command.

        SCPI Syntax:
            ```
            - AFG:OUTPut:STATE {ON|OFF|1|0}
            - AFG:OUTPut:STATE?
            ```

        Info:
            - ``1`` or ON turns on the AFG output state.
            - ``0`` or OFF turns it off.
        """
        return self._state


class AfgOffset(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:OFFSet`` command.

    Description:
        - Sets (or queries) the AFG offset value, in volts.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:OFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:OFFSet?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:OFFSet value`` command.

    SCPI Syntax:
        ```
        - AFG:OFFSet <NR3>
        - AFG:OFFSet?
        ```

    Info:
        - ``<NR3>`` is a floating point number that represents the AFG offset, in volts.
    """


class AfgNoiseaddState(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:NOISEAdd:STATE`` command.

    Description:
        - Sets (or queries) the AFG additive noise state.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:NOISEAdd:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd:STATE?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:NOISEAdd:STATE value`` command.

    SCPI Syntax:
        ```
        - AFG:NOISEAdd:STATE {ON|OFF|1|0}
        - AFG:NOISEAdd:STATE?
        ```

    Info:
        - ``1`` or ON turns on the AFG additive noise state.
        - ``0`` or OFF turns it off.
    """


class AfgNoiseaddPercent(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:NOISEAdd:PERCent`` command.

    Description:
        - Sets (or queries) the AFG additive noise level as a percentage. Minimum is 0.0%, maximum
          is 100.0% and increment is 1.0%.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:NOISEAdd:PERCent?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd:PERCent?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:NOISEAdd:PERCent value`` command.

    SCPI Syntax:
        ```
        - AFG:NOISEAdd:PERCent <NR3>
        - AFG:NOISEAdd:PERCent?
        ```

    Info:
        - ``<NR3>`` is the floating point number that represents the AFG additive noise level as a
          percentage.
    """


class AfgNoiseadd(SCPICmdRead):
    """The ``AFG:NOISEAdd`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:NOISEAdd?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.percent``: The ``AFG:NOISEAdd:PERCent`` command.
        - ``.state``: The ``AFG:NOISEAdd:STATE`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._percent = AfgNoiseaddPercent(device, f"{self._cmd_syntax}:PERCent")
        self._state = AfgNoiseaddState(device, f"{self._cmd_syntax}:STATE")

    @property
    def percent(self) -> AfgNoiseaddPercent:
        """Return the ``AFG:NOISEAdd:PERCent`` command.

        Description:
            - Sets (or queries) the AFG additive noise level as a percentage. Minimum is 0.0%,
              maximum is 100.0% and increment is 1.0%.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:NOISEAdd:PERCent?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd:PERCent?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:NOISEAdd:PERCent value``
              command.

        SCPI Syntax:
            ```
            - AFG:NOISEAdd:PERCent <NR3>
            - AFG:NOISEAdd:PERCent?
            ```

        Info:
            - ``<NR3>`` is the floating point number that represents the AFG additive noise level as
              a percentage.
        """
        return self._percent

    @property
    def state(self) -> AfgNoiseaddState:
        """Return the ``AFG:NOISEAdd:STATE`` command.

        Description:
            - Sets (or queries) the AFG additive noise state.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:NOISEAdd:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd:STATE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:NOISEAdd:STATE value`` command.

        SCPI Syntax:
            ```
            - AFG:NOISEAdd:STATE {ON|OFF|1|0}
            - AFG:NOISEAdd:STATE?
            ```

        Info:
            - ``1`` or ON turns on the AFG additive noise state.
            - ``0`` or OFF turns it off.
        """
        return self._state


class AfgLowlevel(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:LOWLevel`` command.

    Description:
        - This command sets (or queries) the low level value of the output waveform, in volts, when
          using the arbitrary function generator feature.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:LOWLevel?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:LOWLevel?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:LOWLevel value`` command.

    SCPI Syntax:
        ```
        - AFG:LOWLevel <NR3>
        - AFG:LOWLevel?
        ```

    Info:
        - ``<NR3>`` is the floating point number that represents the AFG low level value, in volts.
    """


class AfgHighlevel(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:HIGHLevel`` command.

    Description:
        - This command sets (or queries) the high level value of the output waveform, in volts, when
          using the arbitrary function generator feature.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:HIGHLevel?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:HIGHLevel?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:HIGHLevel value`` command.

    SCPI Syntax:
        ```
        - AFG:HIGHLevel <NR3>
        - AFG:HIGHLevel?
        ```

    Info:
        - ``<NR3>`` is a floating point number that represents the AFG high level value, in volts.
    """


class AfgFunction(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:FUNCtion`` command.

    Description:
        - Sets (or queries) which AFG function to execute.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:FUNCtion?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:FUNCtion?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:FUNCtion value`` command.

    SCPI Syntax:
        ```
        - AFG:FUNCtion {SINE|SQUare|PULSe|RAMP|NOISe|DC|SINC|GAUSsian|LORENtz|ERISe|EDECAy|HAVERSINe|CARDIac|ARBitrary}
        - AFG:FUNCtion?
        ```

    Info:
        - ``SINE``
        - ``SQUare``
        - ``PULSe``
        - ``RAMP``
        - ``NOISe``
        - ``DC`` - The DC level is controlled by ``AFG:OFFSET``.
        - ``SINC`` (Sin(x)/x).
        - ``GAUSsian``
        - ``LORENtz``
        - ``ERISe``
        - ``EDECAy``
        - ``HAVERSINe``
        - ``CARDIac``
        - ``ARBitrary``
    """  # noqa: E501


class AfgFrequency(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:FREQuency`` command.

    Description:
        - Sets (or queries) the AFG frequency, in Hz.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:FREQuency?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:FREQuency?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:FREQuency value`` command.

    SCPI Syntax:
        ```
        - AFG:FREQuency <NR3>
        - AFG:FREQuency?
        ```

    Info:
        - ``<NR3>`` is the floating point number that represents the AFG frequency, in Hz.
    """


class AfgBurstTrigger(SCPICmdWriteNoArguments):
    """The ``AFG:BURSt:TRIGger`` command.

    Description:
        - This command triggers a burst on AFG output.

    Usage:
        - Using the ``.write()`` method will send the ``AFG:BURSt:TRIGger`` command.

    SCPI Syntax:
        ```
        - AFG:BURSt:TRIGger
        ```
    """


class AfgBurstCcount(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:BURSt:CCOUnt`` command.

    Description:
        - This command sets or returns the cycle count for AFG burst mode.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:BURSt:CCOUnt?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:BURSt:CCOUnt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:BURSt:CCOUnt value`` command.

    SCPI Syntax:
        ```
        - AFG:BURSt:CCOUnt <NR1>
        - AFG:BURSt:CCOUnt?
        ```

    Info:
        - ``<NR1>`` is the cycle count.
    """


class AfgBurst(SCPICmdRead):
    """The ``AFG:BURSt`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:BURSt?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:BURSt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.ccount``: The ``AFG:BURSt:CCOUnt`` command.
        - ``.trigger``: The ``AFG:BURSt:TRIGger`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._ccount = AfgBurstCcount(device, f"{self._cmd_syntax}:CCOUnt")
        self._trigger = AfgBurstTrigger(device, f"{self._cmd_syntax}:TRIGger")

    @property
    def ccount(self) -> AfgBurstCcount:
        """Return the ``AFG:BURSt:CCOUnt`` command.

        Description:
            - This command sets or returns the cycle count for AFG burst mode.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:BURSt:CCOUnt?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:BURSt:CCOUnt?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:BURSt:CCOUnt value`` command.

        SCPI Syntax:
            ```
            - AFG:BURSt:CCOUnt <NR1>
            - AFG:BURSt:CCOUnt?
            ```

        Info:
            - ``<NR1>`` is the cycle count.
        """
        return self._ccount

    @property
    def trigger(self) -> AfgBurstTrigger:
        """Return the ``AFG:BURSt:TRIGger`` command.

        Description:
            - This command triggers a burst on AFG output.

        Usage:
            - Using the ``.write()`` method will send the ``AFG:BURSt:TRIGger`` command.

        SCPI Syntax:
            ```
            - AFG:BURSt:TRIGger
            ```
        """
        return self._trigger


class AfgArbitrarySource(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:ARBitrary:SOUrce`` command.

    Description:
        - This command sets or queries the source name for the Arbitrary Waveform. Currently
          supported sources are either waveform file (.wfm) or text file (.csv).

    Usage:
        - Using the ``.query()`` method will send the ``AFG:ARBitrary:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:SOUrce?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:SOUrce value`` command.

    SCPI Syntax:
        ```
        - AFG:ARBitrary:SOUrce <QString>
        - AFG:ARBitrary:SOUrce?
        ```

    Info:
        - ``<QString>`` is the source name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class AfgArbitrary(SCPICmdRead):
    """The ``AFG:ARBitrary`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:ARBitrary?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.source``: The ``AFG:ARBitrary:SOUrce`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._source = AfgArbitrarySource(device, f"{self._cmd_syntax}:SOUrce")

    @property
    def source(self) -> AfgArbitrarySource:
        """Return the ``AFG:ARBitrary:SOUrce`` command.

        Description:
            - This command sets or queries the source name for the Arbitrary Waveform. Currently
              supported sources are either waveform file (.wfm) or text file (.csv).

        Usage:
            - Using the ``.query()`` method will send the ``AFG:ARBitrary:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary:SOUrce?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:ARBitrary:SOUrce value``
              command.

        SCPI Syntax:
            ```
            - AFG:ARBitrary:SOUrce <QString>
            - AFG:ARBitrary:SOUrce?
            ```

        Info:
            - ``<QString>`` is the source name.
        """
        return self._source


class AfgAmplitude(SCPICmdWrite, SCPICmdRead):
    """The ``AFG:AMPLitude`` command.

    Description:
        - Sets (or queries) the AFG amplitude in volts, peak to peak.

    Usage:
        - Using the ``.query()`` method will send the ``AFG:AMPLitude?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG:AMPLitude?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AFG:AMPLitude value`` command.

    SCPI Syntax:
        ```
        - AFG:AMPLitude <NR3>
        - AFG:AMPLitude?
        ```

    Info:
        - ``<NR3>`` is a floating point number that represents the AFG amplitude, peak to peak, in
          volts.
    """


#  pylint: disable=too-many-instance-attributes
class Afg(SCPICmdRead):
    """The ``AFG`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFG?`` query.
        - Using the ``.verify(value)`` method will send the ``AFG?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.amplitude``: The ``AFG:AMPLitude`` command.
        - ``.arbitrary``: The ``AFG:ARBitrary`` command tree.
        - ``.burst``: The ``AFG:BURSt`` command tree.
        - ``.frequency``: The ``AFG:FREQuency`` command.
        - ``.function``: The ``AFG:FUNCtion`` command.
        - ``.highlevel``: The ``AFG:HIGHLevel`` command.
        - ``.lowlevel``: The ``AFG:LOWLevel`` command.
        - ``.noiseadd``: The ``AFG:NOISEAdd`` command tree.
        - ``.offset``: The ``AFG:OFFSet`` command.
        - ``.output``: The ``AFG:OUTPut`` command tree.
        - ``.period``: The ``AFG:PERIod`` command.
        - ``.pulse``: The ``AFG:PULse`` command tree.
        - ``.ramp``: The ``AFG:RAMP`` command tree.
        - ``.square``: The ``AFG:SQUare`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "AFG") -> None:
        super().__init__(device, cmd_syntax)
        self._amplitude = AfgAmplitude(device, f"{self._cmd_syntax}:AMPLitude")
        self._arbitrary = AfgArbitrary(device, f"{self._cmd_syntax}:ARBitrary")
        self._burst = AfgBurst(device, f"{self._cmd_syntax}:BURSt")
        self._frequency = AfgFrequency(device, f"{self._cmd_syntax}:FREQuency")
        self._function = AfgFunction(device, f"{self._cmd_syntax}:FUNCtion")
        self._highlevel = AfgHighlevel(device, f"{self._cmd_syntax}:HIGHLevel")
        self._lowlevel = AfgLowlevel(device, f"{self._cmd_syntax}:LOWLevel")
        self._noiseadd = AfgNoiseadd(device, f"{self._cmd_syntax}:NOISEAdd")
        self._offset = AfgOffset(device, f"{self._cmd_syntax}:OFFSet")
        self._output = AfgOutput(device, f"{self._cmd_syntax}:OUTPut")
        self._period = AfgPeriod(device, f"{self._cmd_syntax}:PERIod")
        self._pulse = AfgPulse(device, f"{self._cmd_syntax}:PULse")
        self._ramp = AfgRamp(device, f"{self._cmd_syntax}:RAMP")
        self._square = AfgSquare(device, f"{self._cmd_syntax}:SQUare")

    @property
    def amplitude(self) -> AfgAmplitude:
        """Return the ``AFG:AMPLitude`` command.

        Description:
            - Sets (or queries) the AFG amplitude in volts, peak to peak.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:AMPLitude?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:AMPLitude?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:AMPLitude value`` command.

        SCPI Syntax:
            ```
            - AFG:AMPLitude <NR3>
            - AFG:AMPLitude?
            ```

        Info:
            - ``<NR3>`` is a floating point number that represents the AFG amplitude, peak to peak,
              in volts.
        """
        return self._amplitude

    @property
    def arbitrary(self) -> AfgArbitrary:
        """Return the ``AFG:ARBitrary`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:ARBitrary?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:ARBitrary?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.source``: The ``AFG:ARBitrary:SOUrce`` command.
        """
        return self._arbitrary

    @property
    def burst(self) -> AfgBurst:
        """Return the ``AFG:BURSt`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:BURSt?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:BURSt?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.ccount``: The ``AFG:BURSt:CCOUnt`` command.
            - ``.trigger``: The ``AFG:BURSt:TRIGger`` command.
        """
        return self._burst

    @property
    def frequency(self) -> AfgFrequency:
        """Return the ``AFG:FREQuency`` command.

        Description:
            - Sets (or queries) the AFG frequency, in Hz.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:FREQuency?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:FREQuency?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:FREQuency value`` command.

        SCPI Syntax:
            ```
            - AFG:FREQuency <NR3>
            - AFG:FREQuency?
            ```

        Info:
            - ``<NR3>`` is the floating point number that represents the AFG frequency, in Hz.
        """
        return self._frequency

    @property
    def function(self) -> AfgFunction:
        """Return the ``AFG:FUNCtion`` command.

        Description:
            - Sets (or queries) which AFG function to execute.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:FUNCtion?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:FUNCtion?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:FUNCtion value`` command.

        SCPI Syntax:
            ```
            - AFG:FUNCtion {SINE|SQUare|PULSe|RAMP|NOISe|DC|SINC|GAUSsian|LORENtz|ERISe|EDECAy|HAVERSINe|CARDIac|ARBitrary}
            - AFG:FUNCtion?
            ```

        Info:
            - ``SINE``
            - ``SQUare``
            - ``PULSe``
            - ``RAMP``
            - ``NOISe``
            - ``DC`` - The DC level is controlled by ``AFG:OFFSET``.
            - ``SINC`` (Sin(x)/x).
            - ``GAUSsian``
            - ``LORENtz``
            - ``ERISe``
            - ``EDECAy``
            - ``HAVERSINe``
            - ``CARDIac``
            - ``ARBitrary``
        """  # noqa: E501
        return self._function

    @property
    def highlevel(self) -> AfgHighlevel:
        """Return the ``AFG:HIGHLevel`` command.

        Description:
            - This command sets (or queries) the high level value of the output waveform, in volts,
              when using the arbitrary function generator feature.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:HIGHLevel?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:HIGHLevel?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:HIGHLevel value`` command.

        SCPI Syntax:
            ```
            - AFG:HIGHLevel <NR3>
            - AFG:HIGHLevel?
            ```

        Info:
            - ``<NR3>`` is a floating point number that represents the AFG high level value, in
              volts.
        """
        return self._highlevel

    @property
    def lowlevel(self) -> AfgLowlevel:
        """Return the ``AFG:LOWLevel`` command.

        Description:
            - This command sets (or queries) the low level value of the output waveform, in volts,
              when using the arbitrary function generator feature.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:LOWLevel?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:LOWLevel?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:LOWLevel value`` command.

        SCPI Syntax:
            ```
            - AFG:LOWLevel <NR3>
            - AFG:LOWLevel?
            ```

        Info:
            - ``<NR3>`` is the floating point number that represents the AFG low level value, in
              volts.
        """
        return self._lowlevel

    @property
    def noiseadd(self) -> AfgNoiseadd:
        """Return the ``AFG:NOISEAdd`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:NOISEAdd?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:NOISEAdd?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.percent``: The ``AFG:NOISEAdd:PERCent`` command.
            - ``.state``: The ``AFG:NOISEAdd:STATE`` command.
        """
        return self._noiseadd

    @property
    def offset(self) -> AfgOffset:
        """Return the ``AFG:OFFSet`` command.

        Description:
            - Sets (or queries) the AFG offset value, in volts.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:OFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OFFSet?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:OFFSet value`` command.

        SCPI Syntax:
            ```
            - AFG:OFFSet <NR3>
            - AFG:OFFSet?
            ```

        Info:
            - ``<NR3>`` is a floating point number that represents the AFG offset, in volts.
        """
        return self._offset

    @property
    def output(self) -> AfgOutput:
        """Return the ``AFG:OUTPut`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:OUTPut?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:OUTPut?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.load``: The ``AFG:OUTPut:LOAd`` command tree.
            - ``.mode``: The ``AFG:OUTPut:MODe`` command.
            - ``.state``: The ``AFG:OUTPut:STATE`` command.
        """
        return self._output

    @property
    def period(self) -> AfgPeriod:
        """Return the ``AFG:PERIod`` command.

        Description:
            - Sets (or queries) the period of the AFG waveform, in seconds.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:PERIod?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:PERIod?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AFG:PERIod value`` command.

        SCPI Syntax:
            ```
            - AFG:PERIod <NR3>
            - AFG:PERIod?
            ```

        Info:
            - ``NR3`` is the floating point number that represents the AFG period value, in seconds.
        """
        return self._period

    @property
    def pulse(self) -> AfgPulse:
        """Return the ``AFG:PULse`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:PULse?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:PULse?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.width``: The ``AFG:PULse:WIDth`` command.
        """
        return self._pulse

    @property
    def ramp(self) -> AfgRamp:
        """Return the ``AFG:RAMP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:RAMP?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:RAMP?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.symmetry``: The ``AFG:RAMP:SYMmetry`` command.
        """
        return self._ramp

    @property
    def square(self) -> AfgSquare:
        """Return the ``AFG:SQUare`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AFG:SQUare?`` query.
            - Using the ``.verify(value)`` method will send the ``AFG:SQUare?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.duty``: The ``AFG:SQUare:DUty`` command.
        """
        return self._square
